----------------------------------------------------------------
--
-- Filename:                fourval.vhd
-- Inhalt:                  Definition einer eigenen 4-wertigen Logik mvl4 mit
--                          entsprechender Aufloesungsfunktion resolved
--                          und Beispiele zur Ueberladung des "="-Operators
--
----------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder, Gunther Lehmann 
-- Letzte Aenderung:        06.03.94
--
----------------------------------------------------------------

PACKAGE fourval IS

   TYPE mvl4 IS ('X', '0', '1', 'Z');               -- vierwertige Logik
   TYPE mvl4_vector IS ARRAY (natural RANGE <>) of mvl4;  

   FUNCTION resolved (a: mvl4_vector) RETURN mvl4;  -- Aufloesungsfunktion

   SUBTYPE mvl4_r IS resolved mvl4;                 -- aufgeloester Typ
   TYPE mvl4_r_vector IS ARRAY (natural RANGE <>) of mvl4_r;

-- FUNCTION "=" (a,b : mvl4) RETURN boolean;        -- [0] : vordef. Operator
   FUNCTION "=" (a,b : mvl4) RETURN bit;            -- [1] : eigene Definition
   FUNCTION "=" (a,b : mvl4) RETURN mvl4;           -- [2] : eigene Definition

END fourval;

----------------------------------------------------------------

PACKAGE BODY fourval IS

  FUNCTION resolved (a: mvl4_vector) RETURN mvl4 IS -- Aufloesungsfunktion
    VARIABLE result : mvl4 := 'Z';
  BEGIN
      FOR m in a'RANGE LOOP
        IF  a(m) = 'X' OR 
           (a(m) = '1' AND result = '0') OR
           (a(m) = '0' AND result = '1') THEN RETURN 'X';
        ELSIF
	   (a(m) = '0' AND result = 'Z') OR
	   (a(m) = '1' AND result = 'Z') THEN
             result := a(m);
        END IF;
      END LOOP;
    RETURN result;
  END resolved;

  FUNCTION "=" (a, b : mvl4) RETURN bit IS            -- [1]
    VARIABLE result : bit := '0';
  BEGIN
    IF (a = '1' AND b = '1') OR
       (a = '0' AND b = '0') OR
       (a = 'X' AND b = 'X') OR
       (a = 'Z' AND b = 'Z') THEN result := '1';
    END IF;
    RETURN result;
  END "=";

  FUNCTION "=" (a, b : mvl4) RETURN mvl4 IS           -- [2]
    VARIABLE result : mvl4 := '0';
  BEGIN
    IF (a = '1' AND b = '1') OR
       (a = '0' AND b = '0') OR
       (a = 'X' AND b = 'X') OR
       (a = 'Z' AND b = 'Z') THEN result := '1';
    END IF;
    RETURN result;
  END "=";

END fourval;

----------------------------------------------------------------

USE work.fourval.ALL;
ENTITY equal IS

  PORT (a,b,c,d : IN  mvl4; 
        w :       OUT mvl4;
        x,y,z :   OUT boolean );

END equal;

----------------------------------------------------------------

ARCHITECTURE behavioral OF equal IS  -- Beispiel zur Ueberladung
                                     -- des "="-Operators

BEGIN
                                     -- vgl. obige Varianten-Nr.
  w <= (a = b) = (c = d);            -- (a [2] b) [2] (c [2] d)
  x <= (a = b) = mvl4'(c = d);       -- (a [2] b) [0] (c [2] d)
  y <= (a = b) = bit'(c = d);        -- (a [1] b) [0] (c [1] d)
  z <= (a = b) = boolean'(c = d);    -- (a [0] b) [0] (c [0] d)

END behavioral;

----------------------------------------------------------------

USE work.fourval.ALL;
ENTITY resolve IS

  PORT (sel : IN positive; x: OUT mvl4_r);

END resolve;

----------------------------------------------------------------

ARCHITECTURE behavioral OF resolve IS  -- Beispiel zur Aufloesungs-
                                       -- funktion
BEGIN

  x <= '0' WHEN sel = 1 ELSE 'Z';
  x <= '1' WHEN sel = 2 ELSE 'X';

END behavioral;

