------------------------------------------------------------------------------
--
-- Filename:                latch87.vhd
-- Inhalt:                  Verhaltensmodell eines D-Latches mit verschiedenen
--                          Architekturen
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder 
-- Letzte Aenderung:        02.02.94
--
------------------------------------------------------------------------------

ENTITY latch IS

  PORT (d, clk : IN bit; q : BUFFER bit) ;
   
END latch ;


------------------------------------------------------------------------------
--
-- concurrent_1: Verwendung von "conditional signal assignment"
--
------------------------------------------------------------------------------

ARCHITECTURE concurrent_1 OF latch IS

BEGIN

  q <= d WHEN clk = '1' ELSE q ;

END concurrent_1 ;


------------------------------------------------------------------------------
--
-- concurrent_2: Verwendung von "selected signal assignment"
--
------------------------------------------------------------------------------

ARCHITECTURE concurrent_2 OF latch IS

BEGIN

  WITH clk select
    q <= d WHEN '1' ,
         q WHEN OTHERS ;

END concurrent_2 ;


------------------------------------------------------------------------------
--
-- concurrent_3 und concurrent_4: im File latch93.vhd
--
------------------------------------------------------------------------------

------------------------------------------------------------------------------
--
-- concurrent_5: Verwendung von "guarded signal assignment"
--
------------------------------------------------------------------------------

ARCHITECTURE concurrent_5 OF latch IS

BEGIN

  q_assignment : BLOCK (clk = '1')

  BEGIN

    q <= GUARDED d ;

  END BLOCK ;

END concurrent_5 ;


------------------------------------------------------------------------------
--
-- sequential_1: Verwendung der IF-Anweisung im Prozess mit sensitivity-list
--
------------------------------------------------------------------------------

ARCHITECTURE sequential_1 OF latch IS

BEGIN

  q_assignment : PROCESS (d, clk)
  BEGIN

    IF clk = '1' THEN
      q <= d ;
    END IF ;

  END PROCESS q_assignment ;

END sequential_1 ;


------------------------------------------------------------------------------
--
-- sequential_2: Verwendung der IF-Anweisung im Prozess mit WAIT-Anweisung
--
------------------------------------------------------------------------------

ARCHITECTURE sequential_2 OF latch IS

BEGIN

  q_assignment : PROCESS
  BEGIN

    IF clk = '1' THEN
      q <= d ;
    END IF ;

    WAIT ON d, clk ;      -- entspricht "sensitivity-list"

  END PROCESS q_assignment ;

END sequential_2 ;


------------------------------------------------------------------------------
--
-- sequential_3: Verwendung der bedingten WAIT-Anweisung
--
------------------------------------------------------------------------------

ARCHITECTURE sequential_3 OF latch IS

BEGIN

  q_assignment : PROCESS
  BEGIN

    q <= d ;
    WAIT ON d, clk until clk = '1' ;    -- ersetzt IF-Anweisung

  END PROCESS q_assignment ;

END sequential_3 ;


------------------------------------------------------------------------------
--
-- sequential_4: Verwendung der CASE-Anweisung
--
------------------------------------------------------------------------------

ARCHITECTURE sequential_4 OF latch IS

BEGIN

  q_assignment : PROCESS (clk, d)
  BEGIN

    CASE clk IS
      WHEN '1'    => q <= d ;
      WHEN OTHERS => NULL ;
    END CASE ;

  END PROCESS q_assignment ;

END sequential_4 ;


