------------------------------------------------------------------------------
--
-- Filename:            addierer.vhd
-- Inhalt:              Verschiedene Modellierungsalternativen 
--                      eines Vier-Bit-Addierers
--
------------------------------------------------------------------------------
--
-- Autor:               Manfred Selz
-- Letzte Aenderung:    16.02.1994
--
------------------------------------------------------------------------------


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;   -- syntheseprogrammspezifisches Package

------------------------------------------------------------------------------

ENTITY addierer IS

  PORT (

    -- std_logic(_vector) anstatt std_ulogic(_vector), weil die ueberladenen
    -- Plus-Funktionen im syntheseprogrammspezifischen Package nur fuer
    -- std_logic(_vector) definiert sind!

    a,b:  IN  std_logic_vector (3 DOWNTO 0);
    cin:  IN  std_logic;
    s:    OUT std_logic_vector (3 DOWNTO 0);
    cout: OUT std_logic );

END addierer;
	
------------------------------------------------------------------------------

ARCHITECTURE zwei_plus OF addierer IS

  SIGNAL temp: std_logic_vector (4 DOWNTO 0);

BEGIN

  -- Auffuellen mit Nullen, damit rechtsbuendige Operanden entstehen

  temp <= ("0" & a) + ("0" & b) + ("0000" & cin);
  cout <= temp(4);
  s <= temp (3 DOWNTO 0);

END zwei_plus;

------------------------------------------------------------------------------

ARCHITECTURE ein_plus OF addierer IS

  SIGNAL temp: std_logic_vector (5 DOWNTO 0);

BEGIN

  -- nur ein Plus-Operator, da das Carry-Signal in die beiden
  -- Operanden durch Concatenation integriert ist

  temp <= ("0" & a & cin) + ("0" & b & "1");
  cout <= temp(5);
  s <= temp (4 DOWNTO 1);

END ein_plus;

------------------------------------------------------------------------------

ARCHITECTURE ripple OF addierer IS

  SIGNAL c: std_logic_vector (3 DOWNTO 0);

BEGIN

  -- logische Gleichungen eines Ripple-Carry-Addierers

  s <= (a XOR b) XOR (c(2 DOWNTO 0) & cin);
  c <= ((a XOR b) AND (c(2 DOWNTO 0) & cin)) OR (a AND b);
  cout <= c(3);

END ripple;

------------------------------------------------------------------------------

ARCHITECTURE cla OF addierer IS

  SIGNAL c: std_logic_vector (2 DOWNTO 0);
  SIGNAL p,g: std_logic_vector (3 DOWNTO 0);

BEGIN

  -- logische Gleichungen eines Carry-Look-Ahhead-Addierers

  p <= a XOR b;
  g <= a AND b;
  s <= p XOR (c & cin);
  c(0) <= g(0) OR (p(0) AND cin);
  c(1) <= g(1) OR (p(1) AND c(0));
  c(2) <= g(2) OR (p(2) AND c(1));
  cout <= g(3) OR (p(3) AND c(2));

END cla;

