------------------------------------------------------------------------------
--
-- Filename:                nand3.vhd
-- Inhalt:                  Zeitabhaengiges Modell eines NAND3-Gatters, welches
--                          die 9-wertige Logik std_logic_1164 verwendet
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder  
-- Letzte Aenderung:        18.01.1994
--
------------------------------------------------------------------------------

LIBRARY    ieee;
USE        ieee.std_logic_1164.all;

------------------------------------------------------------------------------

ENTITY nand3 IS

    GENERIC (tp_lh: time := 190 ps; 
             tp_hl: time := 240 ps);
    PORT (a,b,c:IN  std_ulogic;
          y:    OUT std_ulogic);

END nand3;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF nand3 IS

-- Funktion zur Bestimmung des Minimums zweier Zeiten

    FUNCTION min (x, y: time) RETURN time IS
    BEGIN
        IF y < x THEN
            RETURN y;
        END IF;
        RETURN x;
    END min;

BEGIN

-- Berechnung des Ausgangs nach entsprechender Verzoegerung

    y <= '1' AFTER tp_lh WHEN ( NOT (a AND b AND c) = '1') ELSE
         '0' AFTER tp_hl WHEN ( NOT (a AND b AND c) = '0') ELSE
         'X' AFTER min (tp_lh,tp_hl);

END behavioral;

------------------------------------------------------------------------------

CONFIGURATION nand3_config OF nand3 IS

    FOR behavioral
    END FOR;

END nand3_config;
