------------------------------------------------------------------------------
--
-- Filename:                alu.vhd
-- Inhalt:                  Verhaltensmodell einer arithmetisch-logischen
--                          Einheit (Uebungsaufgabe ALU)
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder  
-- Letzte Aenderung:        20.01.1994
--
------------------------------------------------------------------------------

USE work.alu_pack.all;

------------------------------------------------------------------------------

ENTITY alu IS

    PORT (instruction          : IN command;
          operand_a, operand_b : IN parity_8;
          result               : OUT parity_16);

BEGIN

    check_parity_8(operand_a);  -- Passive
    check_parity_8(operand_b);  -- Prozeduren

END alu ;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF alu IS

BEGIN

    command_execution : PROCESS (instruction, operand_a, operand_b)

        VARIABLE bit_16 : bit_vector (15 DOWNTO 0);

    BEGIN
 
       CASE instruction IS
            WHEN con_ab =>
                bit_16 := operand_a.vector & operand_b.vector;

            WHEN add_ab =>
                bit_16 := int_to_bit16(bit8_to_int(operand_a.vector)
                                      +bit8_to_int(operand_b.vector));

            WHEN add_of =>
                bit_16 := int_to_bit16(bit8_to_int(operand_b.vector)
                                      +bit8_to_int(ram_offset));

            WHEN mlt_ab =>
                bit_16 := int_to_bit16(bit8_to_int(operand_a.vector)
                                      *bit8_to_int(operand_b.vector));

            WHEN sh_l_b =>
                bit_16(15 DOWNTO 8)  := B"0000_0000";                        -- binaer
                bit_16(7 DOWNTO 1)   := operand_b.vector(6 DOWNTO 0);
                bit_16(0)            := operand_b.vector(7);
    
            WHEN sh_r_b =>
                bit_16(15 DOWNTO 8)  := X"00";                               -- hexadezimal
                bit_16(7)            := operand_b.vector(0);
                bit_16(6 DOWNTO 0)   := operand_b.vector(7 DOWNTO 1);

            WHEN sh_l_k =>                                                   -- "concatenated vector"
                bit_16 := (operand_a.vector(6 DOWNTO 0) 
                         & operand_b.vector & operand_a.vector(7));

            WHEN sh_r_k =>                                                   -- "concatenated vector"
                bit_16 := (operand_b.vector(0) & operand_a.vector 
                         & operand_b.vector(7 DOWNTO 1));

            WHEN flip_b =>
                bit_16(15 DOWNTO 8)  := X"00";                               -- hexadezimal
                bit_16(7 DOWNTO 0)   := (operand_b.vector(3 DOWNTO 0) 
                                       & operand_b.vector(7 DOWNTO 4));
            WHEN flip_k =>
                bit_16(15 DOWNTO 12) := operand_b.vector(3 DOWNTO 0);
                bit_16(11 DOWNTO 8)  := operand_b.vector(7 DOWNTO 4);
                bit_16(7  DOWNTO 4)  := operand_a.vector(3 DOWNTO 0);
                bit_16(3  DOWNTO 0)  := operand_a.vector(7 DOWNTO 4);
            WHEN turn_b =>
                FOR i IN 0 TO 7 LOOP
                    bit_16(i) := operand_b.vector(7-i);
                END LOOP;
                bit_16(15 DOWNTO 8) := X"00";                                -- hexadezimal

            WHEN turn_k =>
                FOR i IN 15 DOWNTO 8 LOOP
                    bit_16(i) := operand_b.vector(15-i);
                END LOOP;
                FOR i IN 7 DOWNTO 0 LOOP
                    bit_16(i) := operand_a.vector(7-i);
                END LOOP;
        END CASE;

        result <= (vector => bit_16, parity => create_parity_16(bit_16));

    END PROCESS;

END behavioral;

