------------------------------------------------------------------------------
--
-- Filename:                adder.vhd
-- Aufgabe:                 2.1.2
-- Inhalt:                  Uebungsaufgabe: strukturale Modellierung
--                          Halbaddierer, Volladdierer, 8 Bit Addierer
--                          Verwendung von or2.vhd and2.vhd xor.vhd
--                          (vorher uebersetzen!)
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder 
-- Letzte Aenderung:        04.02.94
--
------------------------------------------------------------------------------

ENTITY halfadder IS

  PORT (sum_a, sum_b : IN bit; sum, carry : OUT bit ) ;
  
END halfadder ;

------------------------------------------------------------------------------

ARCHITECTURE structural OF halfadder IS

  COMPONENT xor2
    PORT (c1, c2 : IN bit; c3 : OUT bit) ;
  END COMPONENT ;

  COMPONENT and2
    PORT (c4, c5 : IN bit; c6 : OUT bit) ;
  END COMPONENT ;

BEGIN

  xor_instance : xor2 PORT MAP (sum_a, sum_b, sum) ;
  and_instance : and2 PORT MAP (sum_a, sum_b, carry) ;

END structural ;

------------------------------------------------------------------------------

CONFIGURATION halfadder_config OF halfadder IS

  FOR structural
    FOR and_instance : and2 USE ENTITY work.and2 (behavioral)
                       PORT MAP (c4, c5, c6);
    END FOR;
    FOR xor_instance : xor2 USE ENTITY work.exor (behavioral)
                       PORT MAP (c1, c2, c3);
    END FOR;
  END FOR;

END halfadder_config ;

------------------------------------------------------------------------------
------------------------------------------------------------------------------

ENTITY fulladder IS

  PORT (in_1, in_2, in_carry : IN bit; sum, carry : OUT bit ) ;
  
END fulladder ;

------------------------------------------------------------------------------

ARCHITECTURE structural OF fulladder IS

  COMPONENT or2
    PORT (c1, c2 : IN bit; c3 : OUT bit) ;
  END COMPONENT ;

  COMPONENT ha
    PORT (c4, c5 : IN bit; c6, c7 : OUT bit) ;
  END COMPONENT ;

  SIGNAL c_intern1, c_intern2, s_intern : bit ;

BEGIN

  ha_instance1 : ha PORT MAP (in_1, in_2, s_intern, c_intern1) ;
  ha_instance2 : ha PORT MAP (s_intern, in_carry, sum, c_intern2) ;
  or_instance  : or2 PORT MAP (c_intern1, c_intern2, carry) ;

END structural ;

------------------------------------------------------------------------------

CONFIGURATION fulladder_config OF fulladder IS

  FOR structural
    FOR or_instance : or2 USE ENTITY work.or2 (behavioral)
                   PORT MAP (c1,c2,c3);
    END FOR;
    FOR all : ha USE ENTITY work.halfadder (structural)
                   PORT MAP (c4, c5, c6, c7);
      FOR structural
        FOR and_instance : and2 USE ENTITY work.and2 (behavioral)
                       PORT MAP (c4, c5, c6);
        END FOR;
        FOR xor_instance : xor2 USE ENTITY work.exor (behavioral)
                       PORT MAP (c1, c2, c3);
        END FOR;
      END FOR;
    END FOR;
  END FOR;

END fulladder_config ;

------------------------------------------------------------------------------
------------------------------------------------------------------------------

ENTITY adder_8 IS

  PORT (in_1, in_2 : IN  bit_vector (7 DOWNTO 0);
        in_carry   : IN  bit;
        sum        : OUT bit_vector (7 DOWNTO 0);
        carry      : OUT bit );
  
END adder_8 ;

------------------------------------------------------------------------------

ARCHITECTURE structural OF adder_8 IS

  COMPONENT fa
    PORT (c1, c2, c3 : IN bit; c4, c5 : OUT bit) ;
  END COMPONENT ;

  SIGNAL c_intern : bit_vector (0 TO 6) ;

BEGIN

  fa_instance_begin : fa PORT MAP (in_1(0), in_2(0), in_carry,
                                   sum(0), c_intern(0) ) ;

  fa_middle : FOR a IN 1 TO 6 GENERATE
    fa_instance_middle : fa PORT MAP (in_1(a), in_2(a), c_intern(a-1),
                                      sum(a), c_intern(a) ) ;
  END GENERATE ;

  fa_instance_end : fa PORT MAP (in_1(7), in_2(7), c_intern(6),
                                 sum(7), carry ) ;

END structural ;

------------------------------------------------------------------------------

CONFIGURATION adder_config OF adder_8 IS

---------------------------------------------------------------
-- vollstaendige hierarchische Konfiguration bis Basismodelle
---------------------------------------------------------------

  FOR structural

    FOR fa_instance_begin : fa USE ENTITY work.fulladder (structural)
             PORT MAP (in_1=>c1,in_2=>c2,in_carry=>c3,sum=>c4,carry=>c5);
      FOR structural
        FOR or_instance : or2 USE ENTITY work.or2 (behavioral)
                 PORT MAP (c1,c2,c3);
        END FOR;
        FOR all : ha USE ENTITY work.halfadder (structural)
                 PORT MAP (c4, c5, c6, c7);
          FOR structural
            FOR and_instance : and2 USE ENTITY work.and2 (behavioral)
                     PORT MAP (c4, c5, c6);
            END FOR;
            FOR xor_instance : xor2 USE ENTITY work.exor (behavioral)
                     PORT MAP (c1, c2, c3);
            END FOR;
          END FOR;
        END FOR;
      END FOR;
    END FOR;

---------------------------------------------------------------
-- Verwendung der Konfiguration des Halbaddierers
---------------------------------------------------------------

    FOR fa_middle  -- GENERATE muss als eine Ebene angegeben werden   

      FOR fa_instance_middle : fa USE ENTITY work.fulladder (structural)
               PORT MAP (in_1=>c1,in_2=>c2,in_carry=>c3,sum=>c4,carry=>c5);
        FOR structural
          FOR or_instance : or2 USE ENTITY work.or2 (behavioral)
                   PORT MAP (c1,c2,c3);
          END FOR;
          FOR all : ha USE CONFIGURATION work.halfadder_config 
                   PORT MAP (c4, c5, c6, c7);
          END FOR;
        END FOR;
      END FOR;

    END FOR;

---------------------------------------------------------------
-- Verwendung der Konfiguration des Volladdierers
---------------------------------------------------------------

    FOR fa_instance_end : fa USE CONFIGURATION work.fulladder_config 
             PORT MAP (in_1=>c1,in_2=>c2,in_carry=>c3,sum=>c4,carry=>c5);
    END FOR;

 END FOR;

END adder_config ;

 

