------------------------------------------------------------------------------
--
-- Filename:                loops.vhd
-- Aufgabe:                 2.1.3
-- Inhalt:                  Beispiele verschiedener Schleifenrealisierungen
--                          Stellenweises Multiplizieren v. 2 Integer-Vektoren
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder 
-- Letzte Aenderung:        07.02.94
--
------------------------------------------------------------------------------

PACKAGE multiply IS

  TYPE int_vector IS ARRAY (integer RANGE <>) OF integer ;

  FUNCTION mult_while (a,b : int_vector(1 TO 8)) RETURN int_vector ;
  FUNCTION mult_for   (a,b : int_vector(1 TO 8)) RETURN int_vector ;
  FUNCTION mult_infty (a,b : int_vector(1 TO 8)) RETURN int_vector ;
  FUNCTION mult_flex  (a,b : int_vector)         RETURN int_vector ;

END multiply ;

------------------------------------------------------------------------------

PACKAGE BODY multiply IS

----------------------------------
-- Verwendung einer WHILE-Schleife
----------------------------------

  FUNCTION mult_while (a,b : int_vector(1 TO 8)) RETURN int_vector IS
    VARIABLE count : integer ;
    VARIABLE result : int_vector (1 TO 8);
  BEGIN
    count := 0;
    WHILE count < 8 LOOP
      count := count + 1;
      result(count) := a(count)*b(count);
    END LOOP;
    RETURN result;
  END mult_while ;

--------------------------------
-- Verwendung einer FOR-Schleife
--------------------------------

  FUNCTION mult_for   (a,b : int_vector(1 TO 8)) RETURN int_vector IS
    VARIABLE result : int_vector (1 TO 8);
  BEGIN
    FOR count IN 1 TO 8 LOOP
      result(count) := a(count)*b(count);
    END LOOP;
    RETURN result;
  END mult_for ;

-----------------------------------
-- Verwendung einer Endlos-Schleife
-- mit EXIT-Anweisung
-----------------------------------

  FUNCTION mult_infty (a,b : int_vector(1 TO 8)) RETURN int_vector IS
    VARIABLE count : integer ;
    VARIABLE result : int_vector (1 TO 8);
  BEGIN
    count := 0;
    LOOP
      count := count + 1;
      result(count) := a(count)*b(count);
      EXIT WHEN count = 8;
    END LOOP;
    RETURN result;
  END mult_infty ;

-------------------------------------------
-- Verwendung einer flexiblen FOR-Schleife:
-- variable Vektorlaenge und Indizierung
-------------------------------------------

  FUNCTION mult_flex (a,b : int_vector) RETURN int_vector IS
    ALIAS a_int : int_vector (1 TO a'LENGTH) IS a;
    ALIAS b_int : int_vector (1 TO b'LENGTH) IS b;
    VARIABLE result : int_vector (1 TO a'LENGTH);
  BEGIN
    ASSERT a'LENGTH = b'LENGTH REPORT "Vectors have different length!"
                               SEVERITY error;
    FOR count IN a_int'RANGE LOOP
      result(count) := a_int(count)*b_int(count);
    END LOOP;
    RETURN result;
  END mult_flex ;

END multiply ;

