------------------------------------------------------------------------------
--
-- Filename:                tab_tb.vhd
-- Inhalt:                  Testumgebung fuer den Telefonanrufbeantworter
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder 
-- Letzte Aenderung:        07.02.94
--
------------------------------------------------------------------------------

ENTITY tab_fsm_tb IS

END tab_fsm_tb;

------------------------------------------------------------------------------

ARCHITECTURE arch_1 OF tab_fsm_tb IS

    SUBTYPE stim IS bit_vector(1 TO 5);

    SIGNAL anruf, hoerer_auf, ansage_fertig, timer_ende, reset, takt : bit;
    SIGNAL timer_start, ansage_abspielen, anruf_aufzeichnen : bit;

    COMPONENT mut_socket
        PORT (pin1, pin2, pin3, pin4, pin5, pin6 : IN  bit; 
              pin7, pin8, pin9                   : OUT bit);
    END COMPONENT;

BEGIN

  model_under_test: mut_socket PORT MAP 
                               (takt, reset, anruf, hoerer_auf, ansage_fertig,
                                timer_ende, timer_start, ansage_abspielen, 
                                anruf_aufzeichnen);

  takt <= NOT takt AFTER 500 us;                   -- Takt: 1 kHz 

  stimuli_generation : PROCESS

  BEGIN

   ---------------------------------------------------
   -- Stimulibeschreibung zum Zeitnullpunkt zugewiesen
   -- ms : fiktive Zeit
   ---------------------------------------------------

    (reset, anruf, hoerer_auf, ansage_fertig, timer_ende) <= 
               stim'("00100") AFTER  0 ms,
               stim'("10100") AFTER  1 ms,        -- RESET-Signal
               stim'("00100") AFTER  10 ms,
               stim'("01100") AFTER  11 ms,       -- Anruf
               stim'("01101") AFTER  100 ms,      -- Timer-Ende
               stim'("01100") AFTER  101 ms,
               stim'("01110") AFTER  200 ms,      -- Ansage_fertig
               stim'("01100") AFTER  201 ms,
               stim'("00100") AFTER  500 ms,      -- Anruf beendet

               stim'("00000") AFTER  600 ms,      -- interaktiv: Hoerer abgenommen
               stim'("00100") AFTER  700 ms,

               stim'("01100") AFTER  1000 ms,     -- Anruf
               stim'("01101") AFTER  1100 ms,     -- Timer-Ende
               stim'("01000") AFTER  1150 ms,     -- Abnehmen waehrend Ansage
               stim'("01100") AFTER  1300 ms,     -- Hoerer wieder auflegen
               stim'("00100") AFTER  1350 ms;     -- Partner legt auch auf

    --------------------------------------------------------------
    -- erwartete Antworten: pruefe 2 ms nach Eintreten der Stimuli
    --------------------------------------------------------------

    WAIT FOR  12 ms; -- absolute Zeit: 12 ms 
    ASSERT (timer_start = '1')       REPORT "timer_start       isn't 1"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR  90 ms; -- absolute Zeit: 102 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '1')  REPORT "ansage_abspielen  isn't 1"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR 100 ms; -- absolute Zeit: 202 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '1') REPORT "anruf_aufzeichnen isn't 1"   SEVERITY note;

    WAIT FOR 300 ms; -- absolute Zeit: 502 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR 100 ms; -- absolute Zeit: 602 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR 100 ms; -- absolute Zeit: 702 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR 300 ms; -- absolute Zeit: 1002 ms 
    ASSERT (timer_start = '1')       REPORT "timer_start       isn't 1"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR 100 ms; -- absolute Zeit: 1102 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '1')  REPORT "ansage_abspielen  isn't 1"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR  50 ms; -- absolute Zeit: 1152 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR 150 ms; -- absolute Zeit: 1302 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT FOR  50 ms; -- absolute Zeit: 1352 ms 
    ASSERT (timer_start = '0')       REPORT "timer_start       isn't 0"   SEVERITY note;
    ASSERT (ansage_abspielen = '0')  REPORT "ansage_abspielen  isn't 0"   SEVERITY note;
    ASSERT (anruf_aufzeichnen = '0') REPORT "anruf_aufzeichnen isn't 0"   SEVERITY note;

    WAIT;                                       -- Ende der Simulation

    END PROCESS;

END arch_1;

------------------------------------------------------------------------------

CONFIGURATION tab_fsm_tb_config OF tab_fsm_tb IS

    FOR arch_1
        FOR model_under_test: mut_socket USE ENTITY work.tab_fsm (behavioral)
            PORT MAP (clk               => pin1,
                      reset             => pin2,
                      anruf             => pin3,
                      hoerer_aufgelegt  => pin4,
                      ansage_fertig     => pin5,
                      timer_ende        => pin6,
                      timer_start       => pin7,
                      ansage_abspielen  => pin8,
                      anruf_aufzeichnen => pin9);
        END FOR;
    END FOR;

END tab_fsm_tb_config;
